from .ipclient import Client, BASE_URL, CLIENT_VERSION, DEFAULT_TIMEOUT
from .httpclient import HTTPClient
from .errors import Error, HTTPError, ResponseCode
from .models import PaginationInfo, Response, Message, Recipient, InboxMessage, Pattern



import sys

import requests
from requests import RequestException
from myippanel.errors import HTTPError, parse_errors
from myippanel.models import Response
from urllib.parse import urljoin
import json


class HTTPClient:
    def __init__(self, apikey, base_url, timeout, client_version="1.0.0"):
        self.apikey = apikey
        self.timeout = timeout
        self.base_url = base_url
        self.client_version = client_version
        self.__supported_status_codes = [200, 201, 204, 400, 401, 403, 404, 405, 422, 500]

    def req(self, method, url, data=None, params=None):
        """
        make http request with prefixed base url, given data and params
        """
        if params is None:
            params = {}

        target_url = urljoin(self.base_url, url)
        user_agent = f"IPPanel/ApiClient/{self.client_version} Python/{sys.hexversion}"
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "apikey": self.apikey,
            "User-Agent": user_agent,
        }
        default_headers = requests.utils.default_headers()
        default_headers.update(headers)

        methods = {
            'DELETE': lambda: requests.delete(target_url, headers=headers, data=json.dumps(data), params=params, timeout=self.timeout),
            'GET': lambda: requests.get(target_url, headers=headers, params=params, timeout=self.timeout),
            'PATCH': lambda: requests.patch(target_url, headers=headers, data=json.dumps(data), timeout=self.timeout),
            'POST': lambda: requests.post(target_url, headers=headers, data=json.dumps(data), timeout=self.timeout),
            'PUT': lambda: requests.put(target_url, headers=headers, data=json.dumps(data), timeout=self.timeout)
        }

        if method not in methods:
            raise ValueError(str(method) + " is not in supported methods")

        try:
            response = methods[method]()

            if response.status_code not in self.__supported_status_codes:
                response.raise_for_status()
        except RequestException as e:
            raise HTTPError(e)

        parsed_response = Response(json.loads(response.content))
        errors = parse_errors(parsed_response)
        
        if isinstance(errors, Exception):
            raise errors

        return parsed_response

    def get(self, url, params=None):
        """
        make http GET request with prefixed base url and given data
        """
        return self.req("GET", url, None, params)

    def post(self, url, data):
        """
        make http POST request with prefixed base url and given data
        """
        return self.req("POST", url, data)



class Base(object):
    """
    base model 
    """

    def from_json(self, data={}):
        for name, value in data.items():
            if name == 'from':
                name = 'sender'
            if hasattr(self, name) and not callable(getattr(self, name)):
                setattr(self, name, value)

    def __repr__(self):
        return str(self.__dict__)


class PaginationInfo(Base):
    """
    response pagination info template
    """

    def __init__(self, data):
        self.total = None
        self.limit = None
        self.page = None
        self.pages = None
        self.prev = None
        self.next = None

        super(PaginationInfo, self).from_json(data)


class Response(Base):
    """
    api response template
    """

    def __init__(self, data):
        self.status = None
        self.code = None
        self.data = None
        self.meta = None
        self.error_message = None

        super(Response, self).from_json(data)

        if "meta" in data:
            self.meta = PaginationInfo(data["meta"])


class Message(Base):
    """
    message object template
    """

    def __init__(self, data):
        self.message_id = None
        self.number = None
        self.message = None
        self.state = None
        self.type = None
        self.valid = None
        self.time = None
        self.time_send = None
        self.recipient_count = None
        self.exit_count = None
        self.part = None
        self.cost = None
        self.return_cost = None
        self.summary = None

        super(Message, self).from_json(data)


class Recipient(Base):
    """
    message recipient object template
    """

    def __init__(self, data):
        self.recipient = None
        self.status = None

        super(Recipient, self).from_json(data)


class InboxMessage(Base):
    """
    inbox message template
    """

    def __init__(self, data):
        self.to = None
        self.message = None
        self.sender = None
        self.created_at = None
        self.type = None

        super(InboxMessage, self).from_json(data)


class Pattern(Base):
    """
    pattern template
    """

    def __init__(self, data):
        self.code = None
        self.status = None
        self.message = None
        self.is_shared = None

        super(Pattern, self).from_json(data)



import enum

from myippanel.models import Response


class ResponseCode(enum.Enum):
    ErrForbidden = 403
    ErrNotFound = 404
    ErrUnprocessableEntity = 422
    ErrInternalServer = 500


class Error(Exception):
    """
    Error template
    """

    def __init__(self, code, message):
        self.code = code
        self.message = message

        super(Error, self).__init__(str(message))


class HTTPError(Exception):
    pass


def parse_errors(response: Response):
    if response.code != 200 or response.error_message != '' and response.error_message is not None:
        return Error(response.code, response.error_message)
    return



from myippanel.httpclient import HTTPClient
from myippanel.models import Message, Recipient, InboxMessage

# base url for api
BASE_URL = "https://api2.ippanel.com/api/v1/"
# default timeout for http client
DEFAULT_TIMEOUT = 30
# client version
CLIENT_VERSION = "2.0.7"


class Client:
    ''' myippanel client class
    '''

    def __init__(self, apikey, http_client=None):
        self.client = http_client or HTTPClient(
            apikey,
            BASE_URL,
            DEFAULT_TIMEOUT,
            CLIENT_VERSION,
        )
        self.apikey = apikey

    def get_credit(self):
        r"""Get authenticated user credit

        :return: :class:`float <float>` object
        :rtype: float
        """
        res = self.client.get("sms/accounting/credit/show")

        try:
            return res.data["credit"]
        except:
            raise ValueError("returned response not valid")

    def send(self, sender, recipients, message, summary):
        r"""Send a message from sender to many recipients.

        :param sender: sender number, string.
        :param recipients: recipients list, list.
        :param message: message to send, string.
        :param summary: description of the message to be logged, string.
        :return: :class:`int <int>` object
        :rtype: int
        """
        res = self.client.post("sms/send/webservice/single", {
            "sender": sender,
            "recipient": recipients,
            "message": message,
            "description": {
                "summary": summary,
                "count_recipient": f"{len(recipients)}"
            },
        })

        try:
            return res.data["message_id"]
        except:
            raise ValueError("returned response not valid")

    def get_message(self, message_id):
        r"""Get a message brief info

        :param message_id: message id, int.
        :return: :class:`Message <Message>` object
        :rtype: models.Message
        """
        res = self.client.get("sms/message/all", {
            'message_id': message_id,
        })

        try:
            return Message(next(item for item in res.data if item['message_id'] == message_id))
            # return Message(res.data[0])
        except:
            raise ValueError("returned response not valid")

    def fetch_statuses(self, message_id, page=0, limit=10):
        r"""Fetch message recipients status

        :param message_id: message id, int.
        :param page: page number(start from 0), int.
        :param limit: fetch limit, int.
        :return: :class:`[]Recipient <[]Recipient>` object
        :rtype: []models.Recipient
        """
        res = self.client.get(f"sms/message/show-recipient/message-id/{message_id}", {
            "page": page,
            "per_page": limit,
        })

        try:
            recipients = []
            for recipient in res.data["deliveries"]:
                recipients.append(Recipient(recipient))

            return recipients, res.meta
        except:
            raise ValueError("returned response not valid")

    def fetch_inbox(self, page=0, limit=10):
        r"""Fetch inbox messages

        :param page: page number(start from 0), int.
        :param limit: fetch limit, int.
        :return: :class:`[]InboxMessage <[]InboxMessage>` object
        :rtype: []models.InboxMessage
        """
        res = self.client.get("/inbox", {
            "page": page,
            "per_page": limit,
        })

        try:
            messages = []
            for message in res.data:
                messages.append(InboxMessage(message))

            return messages, res.meta
        except:
            raise ValueError("returned response not valid")

    def create_pattern(self, pattern, description, variables, delimiter="%", is_shared=False):
        r"""Create a pattern

        :param pattern: pattern schema, string.
        :param description: description of pattern, string.
        :param variables: variable list, string.
        :param delimiter: delimiter of variables in pattern, string.
        :param is_shared: determine that pattern shared or not, bool.
        :return: :class:`int <int>` object
        :rtype: int
        """
        params = {
            "pattern": pattern,
            "description": description,
            "delimiter": delimiter,
            "variable": [],
            "is_shared": is_shared,
        }
        for variable_name, type in variables.items():
            params['variable'].append({'name': variable_name, 'type': type})

        res = self.client.post("sms/pattern/normal/store", params)

        try:
            return res.data[0]["code"]
        except:
            raise ValueError("returned response not valid")

    def send_pattern(self, pattern_code, sender, recipient, values={}):
        r"""Send message with pattern

        :param pattern_code: pattern code, string.
        :param sender: sender number, string.
        :param recipient: recipient number, string.
        :param values: pattern values, dict.
        :return: :class:`int <int>` object
        :rtype: int
        """

        res = self.client.post("sms/pattern/normal/send", {
            "code": pattern_code,
            "sender": sender,
            "recipient": recipient,
            "variable": values,
        })

        try:
            return res.data["message_id"]
        except:
            raise ValueError("returned response not valid")
